package io.gitlab.jfronny.respackopts.gson;

import com.google.gson.*;
import com.google.gson.reflect.TypeToken;
import io.gitlab.jfronny.respackopts.model.condition.*;

import java.lang.reflect.Type;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

public class ConditionDeserializer implements JsonDeserializer<Condition> {
    private static final AndConditionFactory and = new AndConditionFactory();
    private static final Set<CollectingConditionFactory> factories = Set.of(
            and,
            new EqualityConditionFactory(),
            new NorConditionFactory(),
            new OrConditionFactory(),
            new XorConditionFactory()
    );
    private static final Type conditionListType = new TypeToken<List<Condition>>(){}.getType();

    @Override
    public Condition deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context) throws JsonParseException {
        if (json.isJsonObject()) {
            JsonObject jo = json.getAsJsonObject();
            if (jo.size() != 1)
                throw new JsonParseException("More than one key in a condition object");
            for (Map.Entry<String, JsonElement> entry : jo.entrySet()) {
                String name = entry.getKey();
                for (CollectingConditionFactory factory : factories) {
                    if (factory.getNames().contains(name))
                        return factory.build(context.deserialize(entry.getValue(), conditionListType));
                }
                throw new JsonParseException("Unknown condition type: " + name);
            }
        }
        else if (json.isJsonArray()) {
            return and.build(context.deserialize(json, conditionListType));
        }
        else if (json.isJsonPrimitive()) {
            JsonPrimitive pr = json.getAsJsonPrimitive();
            if (pr.isString()) {
                String name = pr.getAsString();
                if (name.toLowerCase(Locale.ROOT).equals("true"))
                    return new BooleanCondition(true);
                if (name.toLowerCase(Locale.ROOT).equals("false"))
                    return new BooleanCondition(false);
                return new RpoBooleanCondition(name);
            }
            else if (pr.isBoolean()) {
                return new BooleanCondition(pr.getAsBoolean());
            }
        }
        throw new JsonParseException("Invalid data type for condition");
    }
}
