package io.gitlab.jfronny.respackopts.gson;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;
import io.gitlab.jfronny.respackopts.Respackopts;
import meteordevelopment.starscript.Script;
import meteordevelopment.starscript.compiler.Compiler;
import meteordevelopment.starscript.compiler.Parser;
import meteordevelopment.starscript.utils.Error;

import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.Map;

public class ScriptDeserializer implements JsonDeserializer<Script> {
    private static final Map<String, Script> compiledScripts = new HashMap<>();

    @Override
    public Script deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context) throws JsonParseException {
        if (json.isJsonPrimitive() && json.getAsJsonPrimitive().isString()) {
            String s = json.getAsJsonPrimitive().getAsString();
            if (compiledScripts.containsKey(s))
                return compiledScripts.get(s);
            Parser.Result result = Parser.parse(s);
            if (result.hasErrors()) {
                for (Error error : result.errors) {
                    Respackopts.LOGGER.error(error);
                }
                throw new JsonParseException("Could not parse script: See errors above");
            }
            Script ss = Compiler.compile(result);
            compiledScripts.put(s, ss);
            return ss;
        }
        else {
            throw new JsonParseException("Could not parse script: Expected string");
        }
    }
}
