package io.gitlab.jfronny.respackopts.gson;

import com.google.gson.Gson;
import com.google.gson.TypeAdapter;
import com.google.gson.TypeAdapterFactory;
import com.google.gson.reflect.TypeToken;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonToken;
import com.google.gson.stream.JsonWriter;
import io.gitlab.jfronny.respackopts.Respackopts;

import java.io.IOException;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

public class SingleEntryListTypeAdapterFactory implements TypeAdapterFactory {
    @Override
    public <T> TypeAdapter<T> create(Gson gson, TypeToken<T> typeToken) {
        Type type = typeToken.getType();
        if (typeToken.getRawType() != List.class
            || !(type instanceof ParameterizedType pt))
            return null;
        if (Respackopts.CONFIG.debugLogs)
            Respackopts.LOGGER.info("Using SingleEntryListTypeAdapter for " + typeToken);
        Type elementType = pt.getActualTypeArguments()[0];
        TypeAdapter<?> elementAdapter = gson.getAdapter(TypeToken.get(elementType));
        return (TypeAdapter<T>) createAdapter(elementAdapter);
    }

    private <T> TypeAdapter<List<T>> createAdapter(final TypeAdapter<T> elementAdapter) {
        return new TypeAdapter<>() {
            @Override
            public void write(JsonWriter out, List<T> value) throws IOException {
                if (value == null) {
                    out.nullValue();
                    return;
                }

                out.beginArray();
                for (T entry : value) {
                    elementAdapter.write(out, entry);
                }
                out.endArray();
            }

            @Override
            public List<T> read(JsonReader in) throws IOException {
                if (in.peek() == JsonToken.NULL) {
                    in.nextNull();
                    return null;
                }

                List<T> list = new ArrayList<>();
                if (in.peek() == JsonToken.BEGIN_ARRAY) {
                    in.beginArray();
                    while (in.hasNext()) {
                        list.add(elementAdapter.read(in));
                    }
                    in.endArray();
                }
                else list.add(elementAdapter.read(in));
                return list;
            }
        };
    }
}
