package io.gitlab.jfronny.respackopts.gson.entry;

import com.google.gson.*;
import io.gitlab.jfronny.respackopts.Respackopts;
import io.gitlab.jfronny.respackopts.model.tree.*;
import io.gitlab.jfronny.respackopts.model.enums.PackReloadType;

import java.lang.reflect.Type;
import java.util.Map;

public class ConfigBranchSerializer implements JsonSerializer<ConfigBranch>, JsonDeserializer<ConfigBranch> {
    @Override
    public JsonElement serialize(ConfigBranch src, Type typeOfSrc, JsonSerializationContext context) {
        JsonObject o = new JsonObject();
        for (Map.Entry<String, ConfigEntry<?>> entry : src.getValue().entrySet()) {
            o.add(entry.getKey(), context.serialize(entry.getValue()));
        }
        return o;
    }

    @Override
    public ConfigBranch deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context) throws JsonParseException {
        if (!json.isJsonObject())
            throw new JsonSyntaxException("ConfigBranch is not an object");
        JsonObject o = json.getAsJsonObject();
        ConfigBranch cbNew = new ConfigBranch();
        for (Map.Entry<String, JsonElement> e : o.entrySet()) {
            JsonElement j = e.getValue();
            String s = e.getKey();
            if (j.isJsonPrimitive()) {
                JsonPrimitive p = j.getAsJsonPrimitive();
                if (p.isBoolean())
                    cbNew.add(s, new ConfigBooleanEntry(p.getAsBoolean()));
                else if (p.isNumber())
                    cbNew.add(s, context.deserialize(j, ConfigNumericEntry.class));
                else if (p.isString())
                    cbNew.add(s, context.deserialize(j, ConfigEnumEntry.class));
            }
            else if (j.isJsonArray())
                cbNew.add(s, context.deserialize(j, ConfigEnumEntry.class));
            else if (j.isJsonObject()) {
                JsonObject jo = j.getAsJsonObject();
                String type;
                if (jo.has("type")) {
                    type = jo.get("type").getAsString();
                }
                else {
                    if (jo.entrySet().stream().allMatch(entry -> "default".equals(entry.getKey()) || "min".equals(entry.getKey()) || "max".equals(entry.getKey()))) {
                        Respackopts.LOGGER.info("No \"type\" property for non-branch entry object, assuming slider");
                        type = "slider";
                    }
                    else {
                        cbNew.add(s, context.deserialize(j, ConfigBranch.class));
                        continue;
                    }
                }
                ConfigEntry<?> entry = switch (type.toLowerCase()) {
                    case "slider", "number", "numeric" -> context.deserialize(j, ConfigNumericEntry.class);
                    case "boolean", "toggle" -> context.deserialize(j, ConfigBooleanEntry.class);
                    case "enum", "select" -> context.deserialize(j, ConfigEnumEntry.class);
                    default -> throw new JsonSyntaxException("Unknown entry type: " + type);
                };
                if (jo.has("default")) {
                    entry.setDefault(context.deserialize(jo.get("default"), entry.getEntryClass()));
                    entry.setValue(context.deserialize(jo.get("default"), entry.getEntryClass()));
                }
                if (jo.has("reloadType")) {
                    entry.setReloadType(context.deserialize(jo.get("reloadType"), PackReloadType.class));
                }
                cbNew.add(s, entry);
            } else if (j.isJsonNull())
                throw new JsonSyntaxException("Unexpected json null");
            else
                throw new JsonSyntaxException("Unexpected json object type");
        }
        return cbNew;
    }
}
