package io.gitlab.jfronny.respackopts.integration;

import io.gitlab.jfronny.respackopts.Respackopts;
import net.fabricmc.loader.api.FabricLoader;

import java.io.IOException;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;

public class DashLoaderCompat {
    public static void requestForceReload() {
        if (!FabricLoader.getInstance().isModLoaded("dashloader"))
            return;
        if (!Respackopts.CONFIG.dashloaderCompat)
            return;
        try {
            if (Respackopts.CONFIG.debugLogs)
                Respackopts.LOGGER.info("Removing DashCache to force dashloader to reload");
            Class<?> loaderClass = Class.forName("net.oskarstrom.dashloader.DashLoader");
            Object loaderInstance = loaderClass.getDeclaredMethod("getInstance").invoke(null);
            Path loaderPath = (Path) loaderClass.getDeclaredMethod("getModBoundDir").invoke(loaderInstance);
            Files.walkFileTree(loaderPath, new SimpleFileVisitor<>() {
                @Override
                public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
                    Files.delete(file);
                    return FileVisitResult.CONTINUE;
                }

                @Override
                public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
                    Files.delete(dir);
                    return FileVisitResult.CONTINUE;
                }
            });
        } catch (Throwable e) {
            Respackopts.LOGGER.error("Failed to remove dashloader data, try disabling dashloaderCompat if this keeps happening", e);
        }
    }
}
