package io.gitlab.jfronny.respackopts.model.tree;

import io.gitlab.jfronny.respackopts.Respackopts;
import io.gitlab.jfronny.respackopts.model.enums.ConfigSyncMode;
import io.gitlab.jfronny.respackopts.model.enums.PackReloadType;
import me.shedaniel.clothconfig2.api.AbstractConfigListEntry;
import meteordevelopment.starscript.value.Value;

public abstract class ConfigEntry<T> {
    private final Class<? super T> entryClass;
    private T defaultValue;
    private T value;
    private PackReloadType reloadType = PackReloadType.Resource;
    protected int version;
    protected ConfigBranch parent;

    public ConfigEntry(Class<? super T> entryClass) {
        this.entryClass = entryClass;
    }

    public String getName() {
        if (parent == null)
            return "";
        String n = parent.getName() + "." + parent.getEntryName(this);
        if (n.startsWith("."))
            n = n.substring(0, 1);
        return n;
    }

    public void setVersion(int version) {
        this.version = version;
    }
    public int getVersion() {
        return version;
    }
    
    public T getValue() {
        if (value == null) {
            if (defaultValue == null) {
                Respackopts.LOGGER.warn("No default value or current value set for entry, returning null in " + getName());
                return null;
            }
            value = getDefault();
        }
        return value;
    }
    
    public T setValue(T value) {
        if (value != null)
            this.value = value;
        return this.value;
    }
    
    public T getDefault() {
        if (defaultValue == null) {
            defaultValue = getValue();
            Respackopts.LOGGER.warn("No default value set for entry, using current in " + getName());
        }
        return defaultValue;
    }
    
    public T setDefault(T value) {
        if (value != null)
            defaultValue = value;
        return defaultValue;
    }
    
    public void sync(ConfigEntry<T> source, ConfigSyncMode mode) {
        if (mode == ConfigSyncMode.RESPACK_LOAD)
            setDefault(source.getDefault());
        if (mode == ConfigSyncMode.CONF_LOAD)
            setValue(source.getValue());
    }
    
    public void appendString(StringBuilder sb) {
        sb.append(value).append(" (").append(defaultValue).append(")");
    }

    @Override
    public String toString() {
        StringBuilder log = new StringBuilder();
        appendString(log);
        return log.toString();
    }
    
    public abstract ConfigEntry<T> clone();

    public abstract void buildShader(StringBuilder sb, String valueName);

    public abstract Value buildStarscript();

    public abstract AbstractConfigListEntry<?> buildEntry(GuiEntryBuilderParam guiEntryBuilderParam);

    public String getEntryType() {
        return "field";
    }

    public Class<? super T> getEntryClass() {
        return entryClass;
    }

    public PackReloadType getReloadType() {
        return reloadType;
    }

    public void setReloadType(PackReloadType reloadType) {
        this.reloadType = reloadType;
    }
}
