package io.gitlab.jfronny.respackopts.model.tree;

import io.gitlab.jfronny.respackopts.Respackopts;
import io.gitlab.jfronny.respackopts.model.enums.ConfigSyncMode;
import io.gitlab.jfronny.respackopts.util.GuiFactory;
import me.shedaniel.clothconfig2.api.AbstractConfigListEntry;
import meteordevelopment.starscript.value.Value;
import net.minecraft.text.Text;

import java.util.*;
import java.util.function.Function;

public class ConfigEnumEntry extends ConfigEntry<String> {
    public List<String> values = new ArrayList<>();
    private Integer nextValue;

    public ConfigEnumEntry() {
        super(String.class);
    }

    public void setNextValue(int v) {
        nextValue = v;
    }

    @Override
    public String getValue() {
        String v = super.getValue();
        if (v == null) {
            if (nextValue != null && nextValue >= 0 && nextValue < values.size()) {
                v = values.get(nextValue);
                setValue(v);
            }
            else {
                throw new NullPointerException("Could not get value in " + getName());
            }
        }
        return v;
    }

    @Override
    public String getDefault() {
        String v = super.getDefault();
        if (v == null) {
            if (values.size() == 0)
                throw new NullPointerException("Could not get default entry as the entry array is empty in " + getName());
            else
                v = values.get(0);
        }
        return v;
    }

    @Override
    public void sync(ConfigEntry<String> source, ConfigSyncMode mode) {
        super.sync(source, mode);
        ConfigEnumEntry n = (ConfigEnumEntry) source;
        if (mode == ConfigSyncMode.RESPACK_LOAD) {
            if (n.values != null && !n.values.isEmpty())
                values = n.values;
        }
        if (getValue() == null && nextValue != null) {
            if (n.nextValue >= 0 && n.nextValue < values.size()) {
                setValue(values.get(n.nextValue));
            }
            else {
                Respackopts.LOGGER.error("Could not load default value for enum in " + getName());
            }
        }
    }

    @Override
    public void appendString(StringBuilder sb) {
        sb.append(getValue()).append('/').append(getValue()).append(" (").append(getDefault()).append('/').append(getDefault()).append(" of:");
        for (String e : values) {
            sb.append(' ').append(e);
        }
        sb.append(')');
    }

    @Override
    public ConfigEntry<String> clone() {
        ConfigEnumEntry e = new ConfigEnumEntry();
        e.nextValue = nextValue;
        e.values = List.copyOf(values);
        e.setValue(getValue());
        e.setDefault(getDefault());
        return e;
    }

    @Override
    public void buildShader(StringBuilder sb, String valueName) {
        sb.append("\n#define ");
        sb.append(valueName);
        sb.append(' ');
        sb.append(values.indexOf(getValue()));
        for (int i = 0; i < values.size(); i++) {
            String e2 = Respackopts.sanitizeString(values.get(i));
            if (version == 1) {
                sb.append("\n#define ");
                sb.append(valueName);
                sb.append('_');
                sb.append(e2);
                sb.append(' ');
                sb.append(i);
            } else {
                if (e2.equals(getValue())) {
                    sb.append("\n#define ");
                    sb.append(valueName);
                    sb.append('_');
                    sb.append(e2);
                }
            }
        }
    }

    @Override
    public Value buildStarscript() {
        return Value.string(getValue());
    }

    @Override
    public AbstractConfigListEntry<?> buildEntry(GuiEntryBuilderParam guiEntryBuilderParam) {
        Function<String, Text> entryText = s -> GuiFactory.getText(s, ("".equals(guiEntryBuilderParam.translationPrefix()) ? "" : guiEntryBuilderParam.translationPrefix() + ".") + guiEntryBuilderParam.entryName());
        Map<String, String> translationReverseLookup = new HashMap<>();
        for (String value : values) {
            translationReverseLookup.put(entryText.apply(value).getString(), value);
        }
        return guiEntryBuilderParam.entryBuilder().startStringDropdownMenu(guiEntryBuilderParam.name(), entryText.apply(getValue()).getString(), entryText)
                .setSuggestionMode(false)
                .setDefaultValue(entryText.apply(getDefault()).getString())
                .setSelections(translationReverseLookup.keySet())
                .setSaveConsumer(value -> {
                    if (!translationReverseLookup.containsKey(value)) {
                        Respackopts.LOGGER.error("Invalid enum entry selected: " + value);
                        return;
                    }
                    value = translationReverseLookup.get(value);
                    if (!Objects.equals(getValue(), value)) {
                        if (Respackopts.CONFIG.debugLogs) Respackopts.LOGGER.info("ConfigEnumEntry SaveCallback");
                        guiEntryBuilderParam.saveCallback();
                    }
                    setValue(value);
                })
                .setTooltipSupplier(guiEntryBuilderParam.tooltipSupplier())
                .build();
    }
}
