package io.gitlab.jfronny.respackopts.model.tree;

import io.gitlab.jfronny.respackopts.Respackopts;
import io.gitlab.jfronny.respackopts.model.enums.ConfigSyncMode;
import io.gitlab.jfronny.respackopts.model.enums.NumericEntryType;
import io.gitlab.jfronny.respackopts.gson.entry.NumericEntrySerializer;
import me.shedaniel.clothconfig2.api.AbstractConfigListEntry;
import me.shedaniel.clothconfig2.impl.builders.DoubleFieldBuilder;
import meteordevelopment.starscript.value.Value;

import java.util.Objects;

public class ConfigNumericEntry extends ConfigEntry<Double> {
    public Double min = null;
    public Double max = null;
    public final NumericEntryType type;
    
    public ConfigNumericEntry(NumericEntryType type) {
        super(Double.class);
        this.type = type;
        setValue(0d);
    }

    @Override
    public Double setDefault(Double value) {
        if ((min != null && value < min) || (max != null && value > max))
            throw new RuntimeException("Default value (" + value + ") out of range in number entry definition (" + min + "-" + max + ")");
        if (type == NumericEntryType.Slider && NumericEntrySerializer.isOdd(value))
            throw new RuntimeException("Default number value must be a whole number");
        return super.setDefault(value);
    }

    @Override
    public void sync(ConfigEntry<Double> source, ConfigSyncMode mode) {
        super.sync(source, mode);
        ConfigNumericEntry n = (ConfigNumericEntry) source;
        if (mode == ConfigSyncMode.RESPACK_LOAD) {
            if (n.min != null)
                min = n.min;
            if (n.max != null)
                max = n.max;
        }
    }

    @Override
    public void appendString(StringBuilder sb) {
        sb.append(getValue())
                .append(" (")
                .append(getDefault())
                .append(", ")
                .append(min)
                .append("-")
                .append(max)
                .append(")");
    }

    @Override
    public ConfigEntry<Double> clone() {
        ConfigNumericEntry ce = new ConfigNumericEntry(type);
        ce.max = max;
        ce.min = min;
        ce.setValue(getValue());
        ce.setDefault(getDefault());
        return ce;
    }

    @Override
    public void buildShader(StringBuilder sb, String valueName) {
        sb.append("\n#define ");
        sb.append(valueName);
        sb.append(' ');
        sb.append(getValue().toString());
    }

    @Override
    public Value buildStarscript() {
        return Value.number(getValue());
    }

    @Override
    public AbstractConfigListEntry<?> buildEntry(GuiEntryBuilderParam guiEntryBuilderParam) {
        if (type == NumericEntryType.Slider) {
            return guiEntryBuilderParam.entryBuilder().startIntSlider(guiEntryBuilderParam.name(),
                    getValue().intValue(), min.intValue(), max.intValue())
                    .setDefaultValue(getDefault().intValue())
                    .setSaveConsumer(value -> {
                        if (!Objects.equals(getValue(), value.doubleValue())) {
                            if (Respackopts.CONFIG.debugLogs) Respackopts.LOGGER.info("ConfigNumericEntrySlider SaveCallback");
                            guiEntryBuilderParam.saveCallback();
                        }
                        setValue(value.doubleValue());
                    })
                    .setTooltipSupplier(guiEntryBuilderParam.tooltipSupplier())
                    .build();
        }
        else {
            DoubleFieldBuilder builder = guiEntryBuilderParam.entryBuilder().startDoubleField(guiEntryBuilderParam.name(), getValue())
                    .setDefaultValue(getDefault())
                    .setSaveConsumer(value -> {
                        if (!Objects.equals(getValue(), value)) {
                            if (Respackopts.CONFIG.debugLogs) Respackopts.LOGGER.info("ConfigNumericEntryNormal SaveCallback");
                            guiEntryBuilderParam.saveCallback();
                        }
                        setValue(value);
                    })
                    .setTooltipSupplier(guiEntryBuilderParam.tooltipSupplier());
            if (min != null) builder.setMin(min);
            if (max != null) builder.setMax(max);
            return builder.build();
        }
    }

}
